@@ -1,9 +1,9 @@
 /*
- *  $Id: dialog.h,v 1.193 2007/07/04 20:11:26 tom Exp $
+ *  $Id: dialog.h,v 1.213 2010/01/19 10:47:42 tom Exp $
  *
  * dialog.h -- common declarations for all dialog modules
  *
- * Copyright 2000-2006,2007 Thomas E. Dickey
+ * Copyright 2000-2008,2010 Thomas E. Dickey
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU Lesser General Public License, version 2.1
@@ -29,6 +29,10 @@
 
 #include <dlg_config.h>
 
+#ifdef __hpux
+#define __HP_CURSES_COMPAT	/* workaround for getattrs, etc. */
+#endif
+
 #include <sys/types.h>
 #include <fcntl.h>
 #include <unistd.h>
@@ -47,12 +51,15 @@
 #include <ncurses/curses.h>
 #elif defined(HAVE_NCURSES_H)
 #include <ncurses.h>
-#elif defined(ultrix)
-#include <cursesX.h>
 #else
 #include <curses.h>
 #endif
 
+/* most curses.h headers include this, some do not */
+#if defined(HAVE_UNCTRL_H)
+#include <unctrl.h>
+#endif
+
 /* possible conflicts with <term.h> which may be included in <curses.h> */
 #ifdef color_names
 #undef color_names
@@ -89,6 +96,7 @@
 #ifdef __hpux
 #undef ACS_UARROW
 #undef ACS_DARROW
+#undef ACS_BLOCK
 #endif
 
 /*
@@ -185,6 +193,9 @@
 #ifndef ACS_DARROW
 #define ACS_DARROW 'v'
 #endif
+#ifndef ACS_BLOCK
+#define ACS_BLOCK '#'
+#endif
 
 /* these definitions may work for antique versions of curses */
 #ifndef HAVE_GETBEGYX
@@ -331,6 +342,10 @@ extern int dlg_getpary(WINDOW * /*win*/);
 /*
  * Callbacks are used to implement the "background" tailbox.
  */
+struct _dlg_callback;
+
+typedef void (*DIALOG_FREEBACK) (struct _dlg_callback * /* p */);
+
 typedef struct _dlg_callback {
     struct _dlg_callback *next;
     FILE *input;
@@ -338,6 +353,10 @@ typedef struct _dlg_callback {
     bool keep_bg;	/* keep in background, on exit */
     bool bg_task;	/* true if this is background task */
     bool (*handle_getc)(struct _dlg_callback *p, int ch, int fkey, int *result);
+    bool keep_win;	/* true to not erase window on exit */
+    /* data for dlg_add_callback_ref */
+    struct _dlg_callback **caller;
+    DIALOG_FREEBACK freeback;
 } DIALOG_CALLBACK;
 
 typedef struct _dlg_windows {
@@ -358,7 +377,7 @@ typedef struct {
     FILE *screen_output;	/* newterm(), etc. */
     bool screen_initialized;
     bool use_colors;		/* use colors by default? */
-    bool use_scrollbar;		/* RESERVED */
+    bool use_scrollbar;		/* option "--scrollbar" */
     bool use_shadow;		/* shadow dialog boxes by default? */
     bool visit_items;		/* option "--visit-items" */
     char *separate_str;		/* option "--separate-widget string" */
@@ -421,10 +440,20 @@ typedef struct {
     unsigned input_length;	/* nonzero if input_result is allocated */
     /* 1.0-20051207 */
     unsigned formitem_type;	/* DIALOG_FORMITEM.type in dialog_form() */
-    /* 1.0-20070227 */
+    /* 1.1-20070227 */
     bool keep_tite;		/* option "--keep-tite" */
     bool ascii_lines;		/* option "--ascii-lines" */
     bool no_lines;		/* option "--no-lines" */
+    /* 1.1-20070930 */
+    bool nook;			/* option "--no-ok" */
+    /* 1.1-20080727 */
+    bool quoted;		/* option "--quoted" */
+    char *column_header;	/* RESERVED "--column-header" */
+    char *column_separator;	/* option "--column-separator" */
+    char *output_separator;	/* option "--output-separator" */
+    /* 1.1-20100118 */
+    char *date_format;		/* option "--date-format" */
+    char *time_format;		/* option "--time-format" */
 } DIALOG_VARS;
 
 #define USE_ITEM_HELP(s)        (dialog_vars.item_help && (s) != 0)
@@ -459,8 +488,8 @@ typedef struct {
     int hilite;
 #endif
 #ifdef HAVE_RC_FILE
-    char *name;
-    char *comment;
+    const char *name;
+    const char *comment;
 #endif
 } DIALOG_COLORS;
 
@@ -469,7 +498,7 @@ extern DIALOG_COLORS dlg_color_table[];
 /*
  * Function prototypes
  */
-extern char *dialog_version(void);
+extern const char *dialog_version(void);
 
 /* widgets, each in separate files */
 extern int dialog_calendar(const char * /*title*/, const char * /*subtitle*/, int /*height*/, int /*width*/, int /*day*/, int /*month*/, int /*year*/);
@@ -526,6 +555,7 @@ extern int dlg_menu(const char * /*title*/, const char * /*cprompt*/, int /*heig
 /* arrows.c */
 extern void dlg_draw_arrows(WINDOW * /*dialog*/, int /*top_arrow*/, int /*bottom_arrow*/, int /*x*/, int /*top*/, int /*bottom*/);
 extern void dlg_draw_arrows2(WINDOW * /*dialog*/, int /*top_arrow*/, int /*bottom_arrow*/, int /*x*/, int /*top*/, int /*bottom*/, chtype /*attr*/, chtype /*borderattr*/);
+extern void dlg_draw_scrollbar(WINDOW * /*dialog*/, long /* first_data */, long /* this_data */, long /* next_data */, long /* total_data */, int /* left */, int /* right */, int /*top*/, int /*bottom*/, chtype /*attr*/, chtype /*borderattr*/);
 
 /* buttons.c */
 extern const char ** dlg_exit_label(void);
@@ -548,6 +578,10 @@ extern void dlg_button_layout(const char ** /*labels*/, int * /*limit*/);
 extern void dlg_button_sizes(const char ** /*labels*/, int /*vertical*/, int * /*longest*/, int * /*length*/);
 extern void dlg_draw_buttons(WINDOW * /*win*/, int /*y*/, int /*x*/, const char ** /*labels*/, int /*selected*/, int /*vertical*/, int /*limit*/);
 
+/* columns.c */
+extern void dlg_align_columns(char ** /* target */, int  /* per_row */, int /* num_rows */);
+extern void dlg_free_columns(char ** /* target */, int  /* per_row */, int /* num_rows */);
+
 /* editbox.c */
 extern int dlg_editbox(const char */*title*/, char ***/*list*/, int */*rows*/, int /*height*/, int /*width*/);
 
@@ -578,6 +612,7 @@ extern int dlg_getc(WINDOW * /*win*/, int * /*fkey*/);
 extern int dlg_getc_callbacks(int /*ch*/, int /*fkey*/, int * /*result*/);
 extern int dlg_last_getc(void);
 extern void dlg_add_callback(DIALOG_CALLBACK * /*p*/);
+extern void dlg_add_callback_ref(DIALOG_CALLBACK ** /*p*/, DIALOG_FREEBACK /* cleanup */);
 extern void dlg_flush_getc(void);
 extern void dlg_remove_callback(DIALOG_CALLBACK * /*p*/);
 extern void dlg_killall_bg(int *retval);
@@ -586,6 +621,7 @@ extern void dlg_killall_bg(int *retval);
 extern WINDOW * dlg_new_modal_window(WINDOW * /*parent*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern WINDOW * dlg_new_window(int /*height*/, int /*width*/, int /*y*/, int /*x*/);
 extern WINDOW * dlg_sub_window(WINDOW * /*win*/, int /*height*/, int /*width*/, int /*y*/, int /*x*/);
+extern bool dlg_need_separator(void);
 extern char * dlg_set_result(const char * /*string*/);
 extern char * dlg_strclone(const char * /*cprompt*/);
 extern char * dlg_strempty(void);
@@ -596,12 +632,16 @@ extern int dlg_box_x_ordinate(int /*width*/);
 extern int dlg_box_y_ordinate(int /*height*/);
 extern int dlg_calc_list_width(int /*item_no*/, DIALOG_LISTITEM * /*items*/);
 extern int dlg_calc_listw(int /*item_no*/, char ** /*items*/, int /*group*/);
+extern int dlg_check_scrolled(int /* key */, int /* last */, int /* page */, bool */* show */, int */* offset */);
 extern int dlg_default_item(char ** /*items*/, int /*llen*/);
 extern int dlg_default_listitem(DIALOG_LISTITEM * /*items*/);
 extern int dlg_defaultno_button(void);
 extern int dlg_max_input(int /*max_len*/);
+extern int dlg_print_scrolled(WINDOW * /* win */, const char * /* prompt */, int /* offset */, int /* height */, int /* width */, int /* pauseopt */);
 extern void dlg_add_quoted(char * /*string*/);
 extern void dlg_add_result(const char * /*string*/);
+extern void dlg_add_separator(void);
+extern void dlg_add_string(char * /*string*/);
 extern void dlg_attr_clear(WINDOW * /*win*/, int /*height*/, int /*width*/, chtype /*attr*/);
 extern void dlg_auto_size(const char * /*title*/, const char * /*prompt*/, int * /*height*/, int * /*width*/, int /*boxlines*/, int /*mincols*/);
 extern void dlg_auto_sizefile(const char * /*title*/, const char * /*file*/, int * /*height*/, int * /*width*/, int /*boxlines*/, int /*mincols*/);
@@ -616,11 +656,13 @@ extern void dlg_draw_bottom_box(WINDOW * /*win*/);
 extern void dlg_draw_box(WINDOW * /*win*/, int /*y*/, int /*x*/, int /*height*/, int /*width*/, chtype /*boxchar*/, chtype /*borderchar*/);
 extern void dlg_draw_title(WINDOW *win, const char *title);
 extern void dlg_exit(int /*code*/) GCC_NORETURN;
-extern void dlg_item_help(char * /*txt*/);
+extern void dlg_item_help(const char * /*txt*/);
 extern void dlg_print_autowrap(WINDOW * /*win*/, const char * /*prompt*/, int /*height*/, int /*width*/);
 extern void dlg_print_size(int /*height*/, int /*width*/);
 extern void dlg_print_text(WINDOW * /*win*/, const char * /*txt*/, int /*len*/, chtype * /*attr*/);
 extern void dlg_put_backtitle(void);
+extern void dlg_restore_vars(DIALOG_VARS * /* save */);
+extern void dlg_save_vars(DIALOG_VARS * /* save */);
 extern void dlg_set_focus(WINDOW * /*parent*/, WINDOW * /*win*/);
 extern void dlg_tab_correct_str(char * /*prompt*/);
 extern void dlg_trim_string(char * /*src*/);
@@ -647,10 +689,17 @@ extern int dlg_strcmp(const char * /*a*/, const char * /*b*/);
 #endif
 
 #ifdef HAVE_DLG_TRACE
+#define DLG_TRACE(params) dlg_trace_msg params
+extern void dlg_trace_msg(const char *fmt, ...)
+#ifdef GCC_PRINTF
+    __attribute__((format(printf,1,2)))
+#endif
+;
 extern void dlg_trace_win(WINDOW * /*win*/);
 extern void dlg_trace_chr(int /*ch*/, int /*fkey*/);
 extern void dlg_trace(const char * /*fname*/);
 #else
+#define DLG_TRACE(params) /* nothing */
 #define dlg_trace_win(win) /* nothing */
 #define dlg_trace_chr(ch,fkey) /* nothing */
 #define dlg_trace(fname) /* nothing */
